/*===========================================================================
/
/									ttPage
/								WXgANZX
/
/============================================================================
/ Copyright (C) 1998-2018 Sota. All rights reserved.
/
/ Redistribution and use in source and binary forms, with or without 
/ modification, are permitted provided that the following conditions 
/ are met:
/
/  1. Redistributions of source code must retain the above copyright 
/     notice, this list of conditions and the following disclaimer.
/  2. Redistributions in binary form must reproduce the above copyright 
/     notice, this list of conditions and the following disclaimer in the 
/     documentation and/or other materials provided with the distribution.
/
/ THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR 
/ IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
/ OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
/ IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, 
/ INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
/ BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF 
/ USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
/ ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
/ (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
/ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
/============================================================================*/

#define  STRICT
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <tchar.h>
#include <windowsx.h>
#include <commctrl.h>

#include "common.h"
#include "resource.h"


/*===== vg^Cv =====*/

static void MakeFontData(LOGFONT font, _TCHAR *buf);
static int RestoreFontData(_TCHAR *str, LOGFONT *font);

static void SetRegType(int Type);
static int OpenReg(_TCHAR *Name, void **Handle);
static int CreateReg(_TCHAR *Name, void **Handle);
static int CloseReg(void *Handle);
static int OpenSubKey(void *Parent, _TCHAR *Name, void **Handle);
static int CreateSubKey(void *Parent, _TCHAR *Name, void **Handle);
static int CloseSubKey(void *Handle);
static int DeleteSubKey(void *Handle, _TCHAR *Name);
static int DeleteValue(void *Handle, _TCHAR *Name);
static int ReadIntValueFromReg(void *Handle, _TCHAR *Name, int *Value);
static int WriteIntValueToReg(void *Handle, _TCHAR *Name, int Value);
static int ReadStringFromReg(void *Handle, _TCHAR *Name, _TCHAR *Str, DWORD Size);
static int WriteStringToReg(void *Handle, _TCHAR *Name, _TCHAR *Str);
static int ReadMultiStringFromReg(void *Handle, _TCHAR *Name, _TCHAR *Str, DWORD Size);
static int WriteMultiStringToReg(void *Handle, _TCHAR *Name, _TCHAR *Str);
static int ReadBinaryFromReg(void *Handle, _TCHAR *Name, void *Bin, DWORD Size);
static int WriteBinaryToReg(void *Handle, _TCHAR *Name, void *Bin, int Len);

/*===== OQ =====*/

GLOBALSET	GlobalSet;			/* ݒl */

/*===== [Jȃ[N =====*/

static LOGFONT ttPageLogFont = {
	-15, 0,
	FALSE, FALSE, FW_NORMAL, FALSE, FALSE, FALSE,
	SHIFTJIS_CHARSET, OUT_STRING_PRECIS, CLIP_STROKE_PRECIS,
	DRAFT_QUALITY, FIXED_PITCH, _T("FixedSys")
};

static COLORREF TextColor = RGB(0,0,0);
static COLORREF HighColor = RGB(255,0,0);
static COLORREF BackColor = RGB(255,255,255);
static COLORREF CtrlColor = RGB(0,0,255);
static COLORREF FindColor = RGB(128,128,128);

static int LineNumDisp = NO;
static int LineNumType = LINENUM_WP;

static int ExtSetNumOnLoad = 0;		/* WXg[h̃t@C^Cvʐݒ̐ */


/*-------------------------------------------------------------------------------
Name    :	SaveRegistory
Desc    :	WXg^INIt@Cɐݒlۑ
Param   :	none
Return  :	none
Note    :	
-------------------------------------------------------------------------------*/
void SaveRegistory(void)
{
	void	*hKey3;
	void	*hKey4;
	void	*hKey5;
	_TCHAR	str[256];
	int		i;
	_TCHAR	*buf;

	SetRegType(GlobalSet.RegType);
	if(CreateReg(_T("ttPage"), &hKey3) == SUCCESS)
	{
		WriteIntValueToReg(hKey3, _T("RegVer"), REGISTORY_VER);

		if(CreateSubKey(hKey3, _T("Options"), &hKey4) == SUCCESS)
		{
			WriteIntValueToReg(hKey4, _T("WinPosX"), GlobalSet.WinPosX);
			WriteIntValueToReg(hKey4, _T("WinPosY"), GlobalSet.WinPosY);
			WriteIntValueToReg(hKey4, _T("WinWidth"), GlobalSet.WinWidth);
			WriteIntValueToReg(hKey4, _T("WinHeight"), GlobalSet.WinHeight);
			WriteIntValueToReg(hKey4, _T("Maximize"), GlobalSet.Maximize);

			WriteIntValueToReg(hKey4, _T("PosSave"), GlobalSet.SaveWinPos);
			WriteIntValueToReg(hKey4, _T("SizeSave"), GlobalSet.SaveWinSize);
			WriteIntValueToReg(hKey4, _T("Tbar"), GlobalSet.ToolBar);
			WriteIntValueToReg(hKey4, _T("Sbar"), GlobalSet.StatusBar);
			WriteIntValueToReg(hKey4, _T("Reg"), GlobalSet.RegType);
			WriteStringToReg(hKey4, _T("Quote"), GlobalSet.QuoteStr);

			WriteIntValueToReg(hKey4, _T("Bell"), GlobalSet.ErrBell);
			WriteIntValueToReg(hKey4, _T("HistNum"), GlobalSet.FileHist);
			WriteIntValueToReg(hKey4, _T("Caret"), GlobalSet.DispCaret);
			WriteIntValueToReg(hKey4, _T("SmLine"), GlobalSet.SmoothLine);
			WriteIntValueToReg(hKey4, _T("SmPage"), GlobalSet.SmoothPage);
			WriteIntValueToReg(hKey4, _T("SmRare"), GlobalSet.SmoothRate);

			WriteIntValueToReg(hKey4, _T("Keys"), GlobalSet.AcceleratorDataNum);
			WriteBinaryToReg(hKey4, _T("KeyTable"), GlobalSet.AcceleratorTable, GlobalSet.AcceleratorDataNum * sizeof(ACCEL));

			if((buf = Malloc(((MAX_PATH+1) * sizeof(_TCHAR)) * HISTORY_MAX+1)) != NULL)
			{
				GetAllHistoryFromMenu(buf, HISTORY_MAX);
				WriteMultiStringToReg(hKey4, _T("Hist"), buf);
				free(buf);
			}

			WriteIntValueToReg(hKey4, _T("Caps"), GlobalSet.FindData.CapsFlg);
			WriteIntValueToReg(hKey4, _T("Regx"), GlobalSet.FindData.Regexp);
			WriteMultiStringToReg(hKey4, _T("FindHist"), GlobalSet.FindHist);

			WriteIntValueToReg(hKey4, _T("ExtSet"), GlobalSet.ExtSetNum);
			for(i = 0; i < GlobalSet.ExtSetNum; i++)
			{
				_stprintf(str, _T("Ext%d"), i);
				if(CreateSubKey(hKey4, str, &hKey5) == SUCCESS)
				{
					WriteStringToReg(hKey5, _T("Name"), GlobalSet.ExtSetTbl[i].Name);
					WriteMultiStringToReg(hKey5, _T("Ext"), GlobalSet.ExtSetTbl[i].Exts);
					WriteIntValueToReg(hKey5, _T("Tab"), GlobalSet.ExtSetTbl[i].TabWidth);
					WriteIntValueToReg(hKey5, _T("Fold"), GlobalSet.ExtSetTbl[i].FoldPos);
					WriteIntValueToReg(hKey5, _T("LineSp"), GlobalSet.ExtSetTbl[i].LineSpace);
					WriteIntValueToReg(hKey5, _T("Cr"), GlobalSet.ExtSetTbl[i].CrMark);
					WriteIntValueToReg(hKey5, _T("TabMark"), GlobalSet.ExtSetTbl[i].TabMark);
					WriteIntValueToReg(hKey5, _T("HanSpcMark"), GlobalSet.ExtSetTbl[i].HanSpcMark);
					WriteIntValueToReg(hKey5, _T("ZenSpcMark"), GlobalSet.ExtSetTbl[i].ZenSpcMark);
					WriteIntValueToReg(hKey5, _T("LineNum"), GlobalSet.ExtSetTbl[i].LineNumDisp);
					WriteIntValueToReg(hKey5, _T("LineType"), GlobalSet.ExtSetTbl[i].LineNumType);
					WriteIntValueToReg(hKey5, _T("FoldRight"), GlobalSet.ExtSetTbl[i].FoldRight);

					MakeFontData(GlobalSet.ExtSetTbl[i].Font, str);
					WriteStringToReg(hKey5, _T("Font"), str);

					WriteBinaryToReg(hKey5, _T("TextCol"), &GlobalSet.ExtSetTbl[i].TextColor, sizeof(COLORREF));
					WriteBinaryToReg(hKey5, _T("HighCol"), &GlobalSet.ExtSetTbl[i].HighColor, sizeof(COLORREF));
					WriteBinaryToReg(hKey5, _T("BackCol"), &GlobalSet.ExtSetTbl[i].BackColor, sizeof(COLORREF));
					WriteBinaryToReg(hKey5, _T("FindCol"), &GlobalSet.ExtSetTbl[i].FindColor, sizeof(COLORREF));
					WriteBinaryToReg(hKey5, _T("CtrlCol"), &GlobalSet.ExtSetTbl[i].CtrlColor, sizeof(COLORREF));
					WriteBinaryToReg(hKey5, _T("CrCol"), &GlobalSet.ExtSetTbl[i].CrColor, sizeof(COLORREF));
					WriteBinaryToReg(hKey5, _T("TabCol"), &GlobalSet.ExtSetTbl[i].TabColor, sizeof(COLORREF));
					WriteBinaryToReg(hKey5, _T("HanSpcCol"), &GlobalSet.ExtSetTbl[i].HanSpcColor, sizeof(COLORREF));
					WriteBinaryToReg(hKey5, _T("ZenSpcCol"), &GlobalSet.ExtSetTbl[i].ZenSpcColor, sizeof(COLORREF));

					WriteIntValueToReg(hKey5, _T("ToolType"), GlobalSet.ExtSetTbl[i].ToolType);
					WriteIntValueToReg(hKey5, _T("EditClose"), GlobalSet.ExtSetTbl[i].EditAndClose);
					WriteIntValueToReg(hKey5, _T("EditReload"), GlobalSet.ExtSetTbl[i].EditAutoReLoad);
					WriteStringToReg(hKey5, _T("Editor"), GlobalSet.ExtSetTbl[i].EditorName);
					WriteStringToReg(hKey5, _T("EdOptEUC"), GlobalSet.ExtSetTbl[i].EditOptEUC);
					WriteStringToReg(hKey5, _T("EdOptJIS"), GlobalSet.ExtSetTbl[i].EditOptJIS);
					WriteStringToReg(hKey5, _T("EdOptSJIS"), GlobalSet.ExtSetTbl[i].EditOptSJIS);
					WriteStringToReg(hKey5, _T("EdOptUTF16"), GlobalSet.ExtSetTbl[i].EditOptUTF16);
					WriteStringToReg(hKey5, _T("EdOptUTF16BE"), GlobalSet.ExtSetTbl[i].EditOptUTF16BE);
					WriteStringToReg(hKey5, _T("EdOptUTF8"), GlobalSet.ExtSetTbl[i].EditOptUTF8);
					WriteStringToReg(hKey5, _T("EdOptUTF7"), GlobalSet.ExtSetTbl[i].EditOptUTF7);

					/* ȉ̃IvV͂ȂȂ */
					RegDeleteKey(hKey5, _T("EditSJIS"));

					CloseSubKey(hKey5);
				}
			}
			/* svɂȂt@C^Cvʐݒ폜 */
			for(; i < ExtSetNumOnLoad; i++)
			{
				_stprintf(str, _T("Ext%d"), i);
				if(RegDeleteKey(hKey4, str) != ERROR_SUCCESS)
				{
					break;
				}
			}

			/* ȉ̃IvV͂ȂȂ */
			RegDeleteKey(hKey4, _T("UI"));

			CloseSubKey(hKey4);
		}
		CloseReg(hKey3);
	}
	return;
}


/*-------------------------------------------------------------------------------
Name    :	LoadRegistory
Desc    :	WXg^INIt@CݒlĂяo
Param   :	history		[out] qXg
Return  :	Xe[^X (YES/NO=ݒ肪o^ĂȂ)
Note    :	
-------------------------------------------------------------------------------*/
int LoadRegistory(_TCHAR *history)
{
	void	*hKey3;
	void	*hKey4;
	void	*hKey5;
	_TCHAR	str[256];
	int		sts;
	int		i;
	int		Ver;
	_TCHAR	tmpExts[EXT_EXT_LEN+1];
	_TCHAR	*get;
	_TCHAR	*put;

	sts = NO;

	*history = NUL;

	SetRegType(REGTYPE_INI);
	if((i = OpenReg(_T("ttPage"), &hKey3)) != SUCCESS)
	{
		SetRegType(REGTYPE_REG);
		i = OpenReg(_T("ttPage"), &hKey3);
	}

	if(i == SUCCESS)
	{
		Ver = 0;
		ReadIntValueFromReg(hKey3, _T("RegVer"), &Ver);

		sts = YES;
		if(OpenSubKey(hKey3, _T("Options"), &hKey4) == SUCCESS)
		{
			ReadIntValueFromReg(hKey4, _T("WinPosX"), &GlobalSet.WinPosX);
			ReadIntValueFromReg(hKey4, _T("WinPosY"), &GlobalSet.WinPosY);
			ReadIntValueFromReg(hKey4, _T("WinWidth"), &GlobalSet.WinWidth);
			ReadIntValueFromReg(hKey4, _T("WinHeight"), &GlobalSet.WinHeight);
			ReadIntValueFromReg(hKey4, _T("Maximize"), &GlobalSet.Maximize);

			ReadIntValueFromReg(hKey4, _T("PosSave"), &GlobalSet.SaveWinPos);
			GlobalSet.SaveWinSize = GlobalSet.SaveWinPos;	// V0.95ȑÔ߂
			ReadIntValueFromReg(hKey4, _T("SizeSave"), &GlobalSet.SaveWinSize);
			ReadIntValueFromReg(hKey4, _T("Tbar"), &GlobalSet.ToolBar);
			ReadIntValueFromReg(hKey4, _T("Sbar"), &GlobalSet.StatusBar);
			ReadIntValueFromReg(hKey4, _T("Reg"), &GlobalSet.RegType);
			ReadStringFromReg(hKey4, _T("Quote"), GlobalSet.QuoteStr, QUOTE_MAXLEN+1);

			ReadIntValueFromReg(hKey4, _T("Bell"), &GlobalSet.ErrBell);
			ReadIntValueFromReg(hKey4, _T("HistNum"), &GlobalSet.FileHist);
			ReadIntValueFromReg(hKey4, _T("Caret"), &GlobalSet.DispCaret);
			ReadIntValueFromReg(hKey4, _T("SmLine"), &GlobalSet.SmoothLine);
			ReadIntValueFromReg(hKey4, _T("SmPage"), &GlobalSet.SmoothPage);
			ReadIntValueFromReg(hKey4, _T("SmRare"), &GlobalSet.SmoothRate);

			if(Ver < REGISTORY_VER_KEY)
			{
				int ui;
				ReadIntValueFromReg(hKey4, _T("UI"), &ui);
				CopyPresetAppKeyTable(ui);
			}
			else
			{
				ReadIntValueFromReg(hKey4, _T("Keys"), &GlobalSet.AcceleratorDataNum);
				GlobalSet.AcceleratorDataMax = GlobalSet.AcceleratorDataNum;
				GlobalSet.AcceleratorTable = Malloc(GlobalSet.AcceleratorDataNum * sizeof(ACCEL));
				ReadBinaryFromReg(hKey4, _T("KeyTable"), GlobalSet.AcceleratorTable, GlobalSet.AcceleratorDataNum * sizeof(ACCEL));
			}

			ReadMultiStringFromReg(hKey4, _T("Hist"), history, (MAX_PATH+1)*HISTORY_MAX+1);

			ReadIntValueFromReg(hKey4, _T("Caps"), &GlobalSet.FindData.CapsFlg);
			ReadIntValueFromReg(hKey4, _T("Regx"), &GlobalSet.FindData.Regexp);
			ReadMultiStringFromReg(hKey4, _T("FindHist"), GlobalSet.FindHist, FINDHIST_NUM*(FINDSTR_LEN+1)+1);

			/* ȉ̐ݒ̓t@C^CvʂɈڂꂽAłȂo[W */
			/* ȍ~ɔāAΓǂݏoĂ */
			if(ReadStringFromReg(hKey4, _T("Font"), str, 256) == SUCCESS)
			{
				RestoreFontData(str, &ttPageLogFont);
			}

			ReadBinaryFromReg(hKey4, _T("TextCol"), &TextColor, sizeof(COLORREF));
			ReadBinaryFromReg(hKey4, _T("HighCol"), &HighColor, sizeof(COLORREF));
			ReadBinaryFromReg(hKey4, _T("BackCol"), &BackColor, sizeof(COLORREF));
			ReadBinaryFromReg(hKey4, _T("CtrlCol"), &CtrlColor, sizeof(COLORREF));
			ReadBinaryFromReg(hKey4, _T("FindCol"), &FindColor, sizeof(COLORREF));

			ReadIntValueFromReg(hKey4, _T("LineNum"), &LineNumDisp);
			ReadIntValueFromReg(hKey4, _T("LineType"), &LineNumType);

			/* t@C^Cvʂ̐ݒ */
			ReadIntValueFromReg(hKey4, _T("ExtSet"), &GlobalSet.ExtSetNum);
			ExtSetNumOnLoad = GlobalSet.ExtSetNum;
			GlobalSet.ExtSetTbl = Malloc(GlobalSet.ExtSetNum * sizeof(EXTSET));
			for(i = 0; i < GlobalSet.ExtSetNum; i++)
			{
				/* ftHgl[h */
				LoadDefaultExtSet(&GlobalSet.ExtSetTbl[i]);

				_stprintf(str, _T("Ext%d"), i);
				if(OpenSubKey(hKey4, str, &hKey5) == SUCCESS)
				{

					GlobalSet.ExtSetTbl[i].FoldRight = NO;

					/* zFAtHgAsԍ̐ݒ肪t@C^CvʂłȂo[W*/
					/* ̐ݒ̓ǂݏoɔ */
					GlobalSet.ExtSetTbl[i].TextColor = TextColor;
					GlobalSet.ExtSetTbl[i].HighColor = HighColor;
					GlobalSet.ExtSetTbl[i].BackColor = BackColor;
					GlobalSet.ExtSetTbl[i].CtrlColor = CtrlColor;
					GlobalSet.ExtSetTbl[i].FindColor = FindColor;
					memcpy(&GlobalSet.ExtSetTbl[i].Font, &ttPageLogFont, sizeof(LOGFONT));
					GlobalSet.ExtSetTbl[i].LineNumDisp = LineNumDisp;
					GlobalSet.ExtSetTbl[i].LineNumType = LineNumType;

					ReadStringFromReg(hKey5, _T("Name"), GlobalSet.ExtSetTbl[i].Name, EXT_NAME_LEN+1);

					if(Ver >= REGISTORY_VER_EXT)
					{
						ReadMultiStringFromReg(hKey5, _T("Ext"), GlobalSet.ExtSetTbl[i].Exts, EXT_EXT_LEN+1);
					}
					else
					{
						/* gqݒt@CɕύX */
						/* ext  *.ext */
						ReadMultiStringFromReg(hKey5, _T("Ext"), tmpExts, EXT_EXT_LEN+1);
						get = tmpExts;
						put = GlobalSet.ExtSetTbl[i].Exts;
						while(*get != NUL)
						{
							_tcscpy(put, _T("*."));
							_tcscpy(put+2, get);
							put += _tcslen(put) + 1;
							get += _tcslen(get) + 1;
						}
					}

					ReadIntValueFromReg(hKey5, _T("Tab"), &GlobalSet.ExtSetTbl[i].TabWidth);
					ReadIntValueFromReg(hKey5, _T("Fold"), &GlobalSet.ExtSetTbl[i].FoldPos);
					ReadIntValueFromReg(hKey5, _T("LineSp"), &GlobalSet.ExtSetTbl[i].LineSpace);
					ReadIntValueFromReg(hKey5, _T("Cr"), &GlobalSet.ExtSetTbl[i].CrMark);
					ReadIntValueFromReg(hKey5, _T("TabMark"), &GlobalSet.ExtSetTbl[i].TabMark);
					ReadIntValueFromReg(hKey5, _T("HanSpcMark"), &GlobalSet.ExtSetTbl[i].HanSpcMark);
					ReadIntValueFromReg(hKey5, _T("ZenSpcMark"), &GlobalSet.ExtSetTbl[i].ZenSpcMark);
					ReadIntValueFromReg(hKey5, _T("LineNum"), &GlobalSet.ExtSetTbl[i].LineNumDisp);
					ReadIntValueFromReg(hKey5, _T("LineType"), &GlobalSet.ExtSetTbl[i].LineNumType);
					ReadIntValueFromReg(hKey5, _T("FoldRight"), &GlobalSet.ExtSetTbl[i].FoldRight);

					if(ReadStringFromReg(hKey5, _T("Font"), str, 256) == SUCCESS)
					{
						RestoreFontData(str, &GlobalSet.ExtSetTbl[i].Font);
					}

					ReadBinaryFromReg(hKey5, _T("TextCol"), &GlobalSet.ExtSetTbl[i].TextColor, sizeof(COLORREF));
					ReadBinaryFromReg(hKey5, _T("HighCol"), &GlobalSet.ExtSetTbl[i].HighColor, sizeof(COLORREF));
					ReadBinaryFromReg(hKey5, _T("BackCol"), &GlobalSet.ExtSetTbl[i].BackColor, sizeof(COLORREF));
					ReadBinaryFromReg(hKey5, _T("FindCol"), &GlobalSet.ExtSetTbl[i].FindColor, sizeof(COLORREF));
					ReadBinaryFromReg(hKey5, _T("CtrlCol"), &GlobalSet.ExtSetTbl[i].CtrlColor, sizeof(COLORREF));
					GlobalSet.ExtSetTbl[i].CrColor = GlobalSet.ExtSetTbl[i].CtrlColor;
					GlobalSet.ExtSetTbl[i].TabColor = GlobalSet.ExtSetTbl[i].CtrlColor;
					GlobalSet.ExtSetTbl[i].HanSpcColor = GlobalSet.ExtSetTbl[i].CtrlColor;
					GlobalSet.ExtSetTbl[i].ZenSpcColor = GlobalSet.ExtSetTbl[i].CtrlColor;
					ReadBinaryFromReg(hKey5, _T("CrCol"), &GlobalSet.ExtSetTbl[i].CrColor, sizeof(COLORREF));
					ReadBinaryFromReg(hKey5, _T("TabCol"), &GlobalSet.ExtSetTbl[i].TabColor, sizeof(COLORREF));
					ReadBinaryFromReg(hKey5, _T("HanSpcCol"), &GlobalSet.ExtSetTbl[i].HanSpcColor, sizeof(COLORREF));
					ReadBinaryFromReg(hKey5, _T("ZenSpcCol"), &GlobalSet.ExtSetTbl[i].ZenSpcColor, sizeof(COLORREF));

					if(Ver >= REGISTORY_VER_TOOL)
					{
						ReadIntValueFromReg(hKey5, _T("ToolType"), &GlobalSet.ExtSetTbl[i].ToolType);
						ReadIntValueFromReg(hKey5, _T("EditClose"), &GlobalSet.ExtSetTbl[i].EditAndClose);
						ReadIntValueFromReg(hKey5, _T("EditReload"), &GlobalSet.ExtSetTbl[i].EditAutoReLoad);
						ReadStringFromReg(hKey5, _T("Editor"), GlobalSet.ExtSetTbl[i].EditorName, MAX_PATH+1 - EDITOR_OPT_LEN);
						ReadStringFromReg(hKey5, _T("EdOptEUC"), GlobalSet.ExtSetTbl[i].EditOptEUC, EDITOR_OPT_LEN+1);
						ReadStringFromReg(hKey5, _T("EdOptJIS"), GlobalSet.ExtSetTbl[i].EditOptJIS, EDITOR_OPT_LEN+1);
						ReadStringFromReg(hKey5, _T("EdOptSJIS"), GlobalSet.ExtSetTbl[i].EditOptSJIS, EDITOR_OPT_LEN+1);
						ReadStringFromReg(hKey5, _T("EdOptUTF16"), GlobalSet.ExtSetTbl[i].EditOptUTF16, EDITOR_OPT_LEN+1);
						ReadStringFromReg(hKey5, _T("EdOptUTF16BE"), GlobalSet.ExtSetTbl[i].EditOptUTF16BE, EDITOR_OPT_LEN+1);
						ReadStringFromReg(hKey5, _T("EdOptUTF8"), GlobalSet.ExtSetTbl[i].EditOptUTF8, EDITOR_OPT_LEN+1);
						ReadStringFromReg(hKey5, _T("EdOptUTF7"), GlobalSet.ExtSetTbl[i].EditOptUTF7, EDITOR_OPT_LEN+1);
					}
					else
					{
						/* ȉhKey4ǂݏoiȑO͌ʐݒł͂Ȃj */
						GlobalSet.ExtSetTbl[i].ToolType = TOOL_TYPE_SPECIFIED;
						ReadIntValueFromReg(hKey4, _T("EditClose"), &GlobalSet.ExtSetTbl[i].EditAndClose);
						ReadIntValueFromReg(hKey4, _T("EditReload"), &GlobalSet.ExtSetTbl[i].EditAutoReLoad);
						ReadStringFromReg(hKey4, _T("Editor"), GlobalSet.ExtSetTbl[i].EditorName, MAX_PATH+1 - EDITOR_OPT_LEN);
						ReadStringFromReg(hKey4, _T("EdOptEUC"), GlobalSet.ExtSetTbl[i].EditOptEUC, EDITOR_OPT_LEN+1);
						ReadStringFromReg(hKey4, _T("EdOptJIS"), GlobalSet.ExtSetTbl[i].EditOptJIS, EDITOR_OPT_LEN+1);
						ReadStringFromReg(hKey4, _T("EdOptSJIS"), GlobalSet.ExtSetTbl[i].EditOptSJIS, EDITOR_OPT_LEN+1);
						if(Ver == 0)
						{
							_tcscat(GlobalSet.ExtSetTbl[i].EditorName, _T(" $K $F"));
						}
					}
					CloseSubKey(hKey5);
				}
			}
			CloseSubKey(hKey4);
		}
		CloseReg(hKey3);
	}
	return(sts);
}


/*-------------------------------------------------------------------------------
Name    :	ClearRegistory
Desc    :	WXg̐ݒlNA
Param   :	none
Return  :	none
Note    :	
-------------------------------------------------------------------------------*/
void ClearRegistory(void)
{
	HKEY	hKey2;
	HKEY	hKey3;
	HKEY	hKey4;
	DWORD	dispos;
	int		i;
	_TCHAR	str[20];

	if(RegCreateKeyEx(HKEY_CURRENT_USER, _T("Software\\Sota"), 0, _T(""), REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey2, &dispos) == ERROR_SUCCESS)
	{
		if(RegCreateKeyEx(hKey2, _T("ttPage"), 0, _T(""), REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey3, &dispos) == ERROR_SUCCESS)
		{
			if(RegCreateKeyEx(hKey3, _T("Options"), 0, _T(""), REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey4, &dispos) == ERROR_SUCCESS)
			{
				for(i = 0; ; i++)
				{
					_stprintf(str, _T("Ext%d"), i);
					if(RegDeleteKey(hKey4, str) != ERROR_SUCCESS)
						break;
				}
				RegCloseKey(hKey4);
			}
			RegDeleteKey(hKey3, _T("Options"));
			RegCloseKey(hKey3);
		}
		RegDeleteKey(hKey2, _T("ttPage"));
		RegCloseKey(hKey2);
	}
	return;
}


/*-------------------------------------------------------------------------------
Name    :	SaveSettingsToFile
Desc    :	ݒt@Cɕۑ
Param   :	none
Return  :	none
Note    :	
-------------------------------------------------------------------------------*/
void SaveSettingsToFile(void)
{
	_TCHAR	tmp[MY_MAX_PATH*2];
	_TCHAR	filename[MY_MAX_PATH+1];

	if(GlobalSet.RegType == REGTYPE_REG)
	{
		_tcscpy(filename, _T("ttPage.reg"));
		if(SelectFileToSave(GetMainHwnd(), filename, MSGJPN_42, MSGJPN_43, _T("reg")) == TRUE)
		{
			_stprintf(tmp, _T("/e \x22%s\x22 HKEY_CURRENT_USER\\Software\\sota\\ttPage"), filename);
			if(ShellExecute(NULL, _T("open"), _T("regedit"), tmp, _T("."), SW_SHOW) <= (HINSTANCE)32)
			{
				DialogBoxParam(GetProgInst(), MAKEINTRESOURCE(common_msg_dlg), GetMainHwnd(), ExeEscMsgDialogProc, (LPARAM)MSGJPN_44);
			}
		}
	}
	else
	{
		_tcscpy(filename, MSGJPN_45);
		if(SelectFileToSave(GetMainHwnd(), filename, MSGJPN_42, MSGJPN_47, _T("ini")) == TRUE)
		{
			CopyFile(AskIniFilePath(), filename, FALSE);
		}
	}
	return;
}


/*-------------------------------------------------------------------------------
Name    :	LoadSettingsFromFile
Desc    :	ݒt@C畜
Param   :	none
Return  :	Xe[^X (YES/NO=Ȃ)
Note    :	
-------------------------------------------------------------------------------*/
int LoadSettingsFromFile(void)
{
	int		ret;
	_TCHAR	tmp[MY_MAX_PATH*2];
	_TCHAR	filename[MY_MAX_PATH+1];

	ret = NO;
	_tcscpy(filename, _T(""));
	if(SelectFileToOpen(GetMainHwnd(), filename, MSGJPN_48, MSGJPN_49, _T("")) == TRUE)
	{
		if((_tcslen(filename) >= 5) && (_tcsicmp(&filename[_tcslen(filename)-4], _T(".reg")) == 0))
		{
			_stprintf(tmp, _T("\x22%s\x22"), filename);

			if(ShellExecute(NULL, _T("open"), _T("regedit"), tmp, _T("."), SW_SHOW) <= (HINSTANCE)32)
			{
				DialogBoxParam(GetProgInst(), MAKEINTRESOURCE(common_msg_dlg), GetMainHwnd(), ExeEscMsgDialogProc, (LPARAM)MSGJPN_44);
			}
			else
			{
				ret = YES;
				/* WXgGfB^Î҂ */
//				WaitForSingleObject(Info.hProcess, INFINITE);
			}
		}
		else if((_tcslen(filename) >= 5) && (_tcsicmp(&filename[_tcslen(filename)-4], _T(".ini")) == 0))
		{
			CopyFile(filename, AskIniFilePath(), FALSE);
			ret = YES;
		}
		else
		{
			DialogBoxParam(GetProgInst(), MAKEINTRESOURCE(common_msg_dlg), GetMainHwnd(), ExeEscMsgDialogProc, (LPARAM)MSGJPN_51);
		}
	}
	return(ret);
}


/*-------------------------------------------------------------------------------
Name    :	MakeFontData
Desc    :	LOGFONTf[^𕶎ɕϊ
Param   :	font	tHgf[^
			buf		obt@
Return  :	none
Note    :	
-------------------------------------------------------------------------------*/
static void MakeFontData(LOGFONT font, _TCHAR *buf)
{
	*buf = NUL;
	_stprintf(buf, _T("%d %d %d %d %d %d %d %d %d %d %d %d %d %s"),
		font.lfHeight, font.lfWidth, font.lfEscapement, font.lfOrientation,
		font.lfWeight, font.lfItalic, font.lfUnderline, font.lfStrikeOut,
		font.lfCharSet, font.lfOutPrecision, font.lfClipPrecision,
		font.lfQuality, font.lfPitchAndFamily, font.lfFaceName);
	return;
}


/*-------------------------------------------------------------------------------
Name    :	RestoreFontData
Desc    :	LOGFONTf[^ɕϊ
Param   :	str		
			font	tHgf[^
Return  :	Xe[^X (SUCCESS/FAIL)
Note    :	
-------------------------------------------------------------------------------*/
static int RestoreFontData(_TCHAR *str, LOGFONT *font)
{
	int		i;
	int		sts;

	sts = FAIL;
	if(_stscanf(str, _T("%d %d %d %d %d %hhd %hhd %hhd %hhd %hhd %hhd %hhd %hhd"),
			&(font->lfHeight), &(font->lfWidth), &(font->lfEscapement), &(font->lfOrientation),
			&(font->lfWeight), &(font->lfItalic), &(font->lfUnderline), &(font->lfStrikeOut),
			&(font->lfCharSet), &(font->lfOutPrecision), &(font->lfClipPrecision),
			&(font->lfQuality), &(font->lfPitchAndFamily)) == 13)
	{
		for(i = 13; i > 0; i--)
		{
			if((str = _tcschr(str, _T(' '))) == NULL)
			{
				break;
			}
			str++;
		}
		if(i == 0)
		{
			_tcscpy(font->lfFaceName, str);
			sts = SUCCESS;
		}
	}

	if(sts == FAIL)
	{
		memset(font, NUL, sizeof(LOGFONT));
	}
	return(sts);
}



/*===== WXgINIt@C̃ANZX ============*/


/*===== INIt@Cp̃WXgf[^ =====*/

typedef struct regdatatbl {
	char	KeyName[80+1];			/* L[ (ANSI) */
	char	ValTbl[REG_SECT_MAX];	/* l̃e[u (ANSI) */
	int		ValLen;					/* lf[^̃oCg */
	int		Mode;					/* L[̃[h */
	struct regdatatbl *Next;
} REGDATATBL;

/*===== vg^Cv =====*/

static BOOL WriteOutRegToFile(REGDATATBL *Pos);
static int ReadInReg(LPTSTR Name, REGDATATBL **Handle);
static int Unicode2AnsiCat(char *buf, LPTSTR str);
static int AnsiCat(char *Src, int Len, char *Dst);
static int StrReadIn(char *Src, int Max, LPTSTR Dst, BOOL multi);
static int StrReadInAnsi(char *Src, int Max, char *Dst);
static char *ScanValue(void *Handle, LPTSTR Name);


/*===== [Jȃ[N =====*/

static int TmpRegType;



/*----- WXg̃^Cvݒ肷 ------------------------------------------
*
*	Parameter
*		int Type : ^Cv (REGTYPE_xxx)
*
*	Return Value
*		int Xe[^X
*----------------------------------------------------------------------------*/

static void SetRegType(int Type)
{
	TmpRegType = Type;
	return;
}


/*----- WXg/INIt@CI[viǂݍ݁j-----------------------
*
*	Parameter
*		LPTSTR Name : WXg
*		void **Handle : nhԂ[N
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int OpenReg(LPTSTR Name, void **Handle)
{
	int Sts;
	_TCHAR Tmp[MY_MAX_PATH+1];

	Sts = FAIL;
	if(TmpRegType == REGTYPE_REG)
	{
		_tcscpy(Tmp, _T("Software\\Sota\\"));
		_tcscat(Tmp, Name);
		if(RegOpenKeyEx(HKEY_CURRENT_USER, Tmp, 0, KEY_READ, (HKEY *)Handle) == ERROR_SUCCESS)
			Sts = SUCCESS;
	}
	else
	{
		if((Sts = ReadInReg(Name, (REGDATATBL **)Handle)) == SUCCESS)
			((REGDATATBL *)(*Handle))->Mode = 0;
	}
	return(Sts);
}


/*----- WXg/INIt@C쐬i݁j---------------------------
*
*	Parameter
*		LPTSTR Name : WXg
*		void **Handle : nhԂ[N
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int CreateReg(LPTSTR Name, void **Handle)
{
	int Sts;
	_TCHAR Tmp[MY_MAX_PATH+1];
	DWORD Dispos;

	Sts = FAIL;
	if(TmpRegType == REGTYPE_REG)
	{
		_tcscpy(Tmp, _T("Software\\Sota\\"));
		_tcscat(Tmp, Name);
		if(RegCreateKeyEx(HKEY_CURRENT_USER, Tmp, 0, _T(""), REG_OPTION_NON_VOLATILE, KEY_CREATE_SUB_KEY | KEY_SET_VALUE, NULL, (HKEY *)Handle, &Dispos) == ERROR_SUCCESS)
			Sts = SUCCESS;
	}
	else
	{
		if((*Handle = Malloc(sizeof(REGDATATBL))) != NULL)
		{
			strcpy(((REGDATATBL *)(*Handle))->KeyName, "");		/* ANSI */
			Unicode2AnsiCat(((REGDATATBL *)(*Handle))->KeyName, Name);
			((REGDATATBL *)(*Handle))->ValLen = 0;
			((REGDATATBL *)(*Handle))->Next = NULL;
			((REGDATATBL *)(*Handle))->Mode = 1;
			Sts = SUCCESS;
		}
	}
	return(Sts);
}


/*----- WXg/INIt@CN[Y ----------------------------------
*
*	Parameter
*		void *Handle : nh
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int CloseReg(void *Handle)
{
	REGDATATBL *Pos;
	REGDATATBL *Next;
	FILE *Strm;

	if(TmpRegType == REGTYPE_REG)
	{
		RegCloseKey(Handle);

		/* INIt@C폜 */
		if((Strm = _tfopen(AskIniFilePath(), _T("rt"))) != NULL)
		{
			fclose(Strm);
			MoveFileToTrashCan(AskIniFilePath());
		}
	}
	else
	{
		if(((REGDATATBL *)Handle)->Mode == 1)
		{
			if(WriteOutRegToFile(Handle) == TRUE)
			{
//				/* WXgNA */
//				ClearRegistory();
			}
		}
		/* e[u폜 */
		Pos = Handle;
		while(Pos != NULL)
		{
			Next = Pos->Next;
			free(Pos);
			Pos = Next;
		}
	}
	return(SUCCESS);
}


/*----- WXgINIt@Cɏ ---------------------------------
*
*	Parameter
*		REGDATATBL *Pos : WXgf[^
*
*	Return Value
*		Ȃ
*----------------------------------------------------------------------------*/

static BOOL WriteOutRegToFile(REGDATATBL *Pos)
{
	FILE	*Strm;
	char	*Disp;
	BOOL	Ret;
	char	*fname;

	Ret = FALSE;
	/* do not use unicode since compatibility */
	fname = AskIniFilePathAnsi();
	if((Strm = fopen(fname, "wt")) != NULL)	/* ANSI */
	{
		fprintf(Strm, MSGJPN_52);
		while(Pos != NULL)
		{
			fprintf(Strm, "\n[%s]\n", Pos->KeyName);

			Disp = Pos->ValTbl;
			while(Disp < (Pos->ValTbl + Pos->ValLen))
			{
				fprintf(Strm, "%s\n", Disp);
				Disp = Disp + strlen(Disp) + 1;	/* ANSI */
			}
			Pos = Pos->Next;
		}
		fclose(Strm);
		Ret = TRUE;
	}
	else
	{
		DialogBoxParam(GetProgInst(), MAKEINTRESOURCE(common_msg_dlg), GetMainHwnd(), ExeEscMsgDialogProc, (LPARAM)MSGJPN_53);
	}
	free(fname);

	return(Ret);
}


/*----- INIt@C烌WXgǂݍ -------------------------------
*
*	Parameter
*		Name : O
*		Handle : nh
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int ReadInReg(LPTSTR Name, REGDATATBL **Handle)
{
	FILE *Strm;
	char *Buf;		/* ANSI */
	char *Tmp;		/* ANSI */
	char *Data;		/* ANSI */
	REGDATATBL *New;
	REGDATATBL *Pos;
	int Sts;
	char	*fname;

	Sts = FAIL;
	*Handle = NULL;

	/* do not use unicode since compatibility */
	fname = AskIniFilePathAnsi();
	if((Strm = fopen(fname, "rt")) != NULL)		/* ANSI */
	{
		if((Buf = Malloc(REG_SECT_MAX)) != NULL)
		{
			while(fgets(Buf, REG_SECT_MAX, Strm) != NULL)	/* ANSI */
			{
				if(*Buf != '#')		/* ANSI */
				{
					if((Tmp = strchr(Buf, '\n')) != NULL)
						*Tmp = NUL;

					if(*Buf == '[')
					{
						if((New = Malloc(sizeof(REGDATATBL))) != NULL)
						{
							if((Tmp = strchr(Buf, ']')) != NULL)	/* ANSI */
								*Tmp = NUL;
							strcpy(New->KeyName, Buf+1);			/* ANSI */
							New->ValLen = 0;
							New->Next = NULL;
							Data = New->ValTbl;
						}
						if(*Handle == NULL)
							*Handle = New;
						else
						{
							Pos = *Handle;
							while(Pos->Next != NULL)
								Pos = Pos->Next;
							Pos->Next = New;
						}
					}
					else if(strlen(Buf) > 0)				/* ANSI */
					{
						strcpy(Data, Buf);					/* ANSI */
						Data += strlen(Buf) + 1;			/* ANSI */
						New->ValLen += strlen(Buf) + 1;		/* ANSI */
					}
				}
			}
			Sts = SUCCESS;
			free(Buf);
		}
		fclose(Strm);
	}
	free(fname);
	return(Sts);
}


/*----- TuL[I[v ------------------------------------------------
*
*	Parameter
*		void *Parent : ẽnh
*		LPTSTR Name : O
*		void **Handle : nhԂ[N
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int OpenSubKey(void *Parent, LPTSTR Name, void **Handle)
{
	int Sts;
	char Key[80];		/* ANSI */
	REGDATATBL *Pos;

	Sts = FAIL;
	if(TmpRegType == REGTYPE_REG)
	{
		if(RegOpenKeyEx(Parent, Name, 0, KEY_READ, (HKEY *)Handle) == ERROR_SUCCESS)
			Sts = SUCCESS;
	}
	else
	{
		strcpy(Key, ((REGDATATBL *)Parent)->KeyName);	/* ANSI */
		strcat(Key, "\\");								/* ANSI */
		Unicode2AnsiCat(Key, Name);
		Pos = Parent;
		while(Pos != NULL)
		{
			if(strcmp(Pos->KeyName, Key) == 0)			/* ANSI */
			{
				*Handle = Pos;
				Sts = SUCCESS;
				break;
			}
			Pos = Pos->Next;
		}
	}
	return(Sts);
}


/*----- TuL[쐬 ----------------------------------------------------
*
*	Parameter
*		void *Parent : ẽnh
*		LPTSTR Name : O
*		void **Handle : nhԂ[N
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int CreateSubKey(void *Parent, LPTSTR Name, void **Handle)
{
	int Sts;
	DWORD Dispos;
	REGDATATBL *Pos;

	Sts = FAIL;
	if(TmpRegType == REGTYPE_REG)
	{
		if(RegCreateKeyEx(Parent, Name, 0, _T(""), REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, (HKEY *)Handle, &Dispos) == ERROR_SUCCESS)
			Sts = SUCCESS;
	}
	else
	{
		if((*Handle = Malloc(sizeof(REGDATATBL))) != NULL)
		{
			strcpy(((REGDATATBL *)(*Handle))->KeyName, "");		/* ANSI */
			strcat(((REGDATATBL *)(*Handle))->KeyName, ((REGDATATBL *)Parent)->KeyName);
			strcat(((REGDATATBL *)(*Handle))->KeyName, "\\");	/* ANSI */
			Unicode2AnsiCat(((REGDATATBL *)(*Handle))->KeyName, Name);
			((REGDATATBL *)(*Handle))->ValLen = 0;
			((REGDATATBL *)(*Handle))->Next = NULL;
			Pos = (REGDATATBL *)Parent;
			while(Pos->Next != NULL)
				Pos = Pos->Next;
			Pos->Next = *Handle;
			Sts = SUCCESS;
		}
	}
	return(Sts);
}


/*----- TuL[N[Y ------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int CloseSubKey(void *Handle)
{
	if(TmpRegType == REGTYPE_REG)
		RegCloseKey(Handle);
	else
	{
		/* Nothing */
	}
	return(SUCCESS);
}


/*----- TuL[폜 ----------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*		LPTSTR Name : O
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int DeleteSubKey(void *Handle, LPTSTR Name)
{
	int Sts;

	Sts = FAIL;
	if(TmpRegType == REGTYPE_REG)
	{
		if(RegDeleteKey(Handle, Name) == ERROR_SUCCESS)
			Sts = SUCCESS;
	}
	else
	{
		Sts = FAIL;
	}
	return(Sts);
}


/*----- l폜 ----------------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*		LPTSTR Name : O
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int DeleteValue(void *Handle, LPTSTR Name)
{
	int Sts;

	Sts = FAIL;
	if(TmpRegType == REGTYPE_REG)
	{
		if(RegDeleteValue(Handle, Name) == ERROR_SUCCESS)
			Sts = SUCCESS;
	}
	else
	{
		Sts = FAIL;
	}
	return(Sts);
}


/*----- INTlǂݍ -------------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*		LPTSTR Name : O
*		int *Value : INTlԂ[N
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int ReadIntValueFromReg(void *Handle, LPTSTR Name, int *Value)
{
	int Sts;
	DWORD Size;
	char *Pos;

	Sts = FAIL;
	if(TmpRegType == REGTYPE_REG)
	{
		Size = sizeof(int);
		if(RegQueryValueEx(Handle, Name, NULL, NULL, (BYTE *)Value, &Size) == ERROR_SUCCESS)
			Sts = SUCCESS;
	}
	else
	{
		if((Pos = ScanValue(Handle, Name)) != NULL)
		{
			*Value = atoi(Pos);
			Sts = SUCCESS;
		}
	}
	return(Sts);
}


/*----- INTl -------------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*		LPTSTR Name : O
*		int Value : INTl
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int WriteIntValueToReg(void *Handle, LPTSTR Name, int Value)
{
	REGDATATBL *Pos;
	char *Data;
	char Tmp[20];	/* ANSI */

	if(TmpRegType == REGTYPE_REG)
		RegSetValueEx(Handle, Name, 0, REG_DWORD, (CONST BYTE *)&Value, sizeof(int));
	else
	{
		Pos = (REGDATATBL *)Handle;
		Data = Pos->ValTbl + Pos->ValLen;

		strcpy(Data, "");					/* ANSI */
		Unicode2AnsiCat(Data, Name);
		strcat(Data, "=");					/* ANSI */
		sprintf(Tmp, "%d", Value);			/* ANSI */
		strcat(Data, Tmp);					/* ANSI */
		Pos->ValLen += strlen(Data) + 1;	/* ANSI */
	}
	return(SUCCESS);
}


/*----- ǂݍ ------------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*		LPTSTR Name : O
*		LPTSTR Str : Ԃ[N
*		DWORD Size : őTCY
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int ReadStringFromReg(void *Handle, LPTSTR Name, LPTSTR Str, DWORD Size)
{
	int Sts;
	char *Pos;

	Sts = FAIL;
	if(TmpRegType == REGTYPE_REG)
	{
		Size = Size * sizeof(_TCHAR);
		if(RegQueryValueEx(Handle, Name, NULL, NULL, (BYTE *)Str, &Size) == ERROR_SUCCESS)
		{
			Size /= sizeof(_TCHAR);
			if(*(Str + Size - 1) != NUL)
				*(Str + Size) = NUL;
			Sts = SUCCESS;
		}
	}
	else
	{
		if((Pos = ScanValue(Handle, Name)) != NULL)
		{
			Size = StrReadIn(Pos, Size, Str, FALSE);
			Sts = SUCCESS;
		}
	}
	return(Sts);
}


/*-----  ------------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*		LPTSTR Name : O
*		LPTSTR Str :
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int WriteStringToReg(void *Handle, LPTSTR Name, LPTSTR Str)
{
	REGDATATBL *Pos;
	char *Data;

	if(TmpRegType == REGTYPE_REG)
		RegSetValueEx(Handle, Name, 0, REG_SZ, (CONST BYTE *)Str, (_tcslen(Str)+1) * sizeof(_TCHAR));
	else
	{
		Pos = (REGDATATBL *)Handle;
		Data = Pos->ValTbl + Pos->ValLen;
		strcpy(Data, "");				/* ANSI */
		Unicode2AnsiCat(Data, Name);
		strcat(Data, "=");				/* ANSI */
		Pos->ValLen += strlen(Data);	/* ANSI */
		Data = Pos->ValTbl + Pos->ValLen;
		Pos->ValLen += Unicode2AnsiCat(Data, Str) + 1;
	}
	return(SUCCESS);
}


/*----- }`ǂݍ ------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*		LPTSTR Name : O
*		LPTSTR Str : Ԃ[N
*		DWORD Size : őTCY
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int ReadMultiStringFromReg(void *Handle, LPTSTR Name, LPTSTR Str, DWORD Size)
{
	int Sts;
	char *Pos;

	Sts = FAIL;
	if(TmpRegType == REGTYPE_REG)
	{
		Size = Size * sizeof(_TCHAR);
		if(RegQueryValueEx(Handle, Name, NULL, NULL, (BYTE *)Str, &Size) == ERROR_SUCCESS)
		{
			Size /= sizeof(_TCHAR);
			if(*(Str + Size - 1) != NUL)
				*(Str + Size) = NUL;
			Sts = SUCCESS;
		}
	}
	else
	{
		if((Pos = ScanValue(Handle, Name)) != NULL)
		{
			Size = StrReadIn(Pos, Size, Str, TRUE);
			Sts = SUCCESS;
		}
	}
	return(Sts);
}


/*----- }` ------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*		LPTSTR Name : O
*		LPTSTR Str : 
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int WriteMultiStringToReg(void *Handle, LPTSTR Name, LPTSTR Str)
{
	REGDATATBL *Pos;
	char *Data;

	if(TmpRegType == REGTYPE_REG)
		RegSetValueEx(Handle, Name, 0, REG_MULTI_SZ, (CONST BYTE *)Str, (AskMultiStrLen(Str)+1) * sizeof(_TCHAR));
	else
	{
		Pos = (REGDATATBL *)Handle;
		Data = Pos->ValTbl + Pos->ValLen;
		strcpy(Data, "");				/* ANSI */
		Unicode2AnsiCat(Data, Name);
		strcat(Data, "=");				/* ANSI */
		Pos->ValLen += strlen(Data);	/* ANSI */
		Data = Pos->ValTbl + Pos->ValLen;
		if(*Str != NUL)
		{
			for(; *Str != NUL;)
			{
				Pos->ValLen += Unicode2AnsiCat(Data, Str);
				strcat(Data, "\\00");
				Pos->ValLen += 3;
				Str = _tcschr(Str, NUL) + 1;
			}
		}
		Pos->ValLen += 1;
	}
	return(SUCCESS);
}


/*----- oCiǂݍ-----------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*		LPTSTR Name : O
*		void *Bin : oCiԂ[N
*		DWORD Size : őTCY
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int ReadBinaryFromReg(void *Handle, LPTSTR Name, void *Bin, DWORD Size)
{
	int Sts;
	char *Pos;

	Sts = FAIL;
	if(TmpRegType == REGTYPE_REG)
	{
		if(RegQueryValueEx(Handle, Name, NULL, NULL, (BYTE *)Bin, &Size) == ERROR_SUCCESS)
			Sts = SUCCESS;
	}
	else
	{
		if((Pos = ScanValue(Handle, Name)) != NULL)
		{
			Size = StrReadInAnsi(Pos, Size, Bin);
			Sts = SUCCESS;
		}
	}
	return(Sts);
}


/*----- oCi ----------------------------------------------------
*
*	Parameter
*		void *Handle : nh
*		LPTSTR Name : O
*		void *Bin : oCi
*		int Len : 
*
*	Return Value
*		int Xe[^X
*			SUCCESS/FAIL
*----------------------------------------------------------------------------*/

static int WriteBinaryToReg(void *Handle, LPTSTR Name, void *Bin, int Len)
{
	REGDATATBL *Pos;
	char *Data;

	if(TmpRegType == REGTYPE_REG)
		RegSetValueEx(Handle, Name, 0, REG_BINARY, (CONST BYTE *)Bin, Len);
	else
	{
		Pos = (REGDATATBL *)Handle;
		Data = Pos->ValTbl + Pos->ValLen;
		strcpy(Data, "");				/* ANSI */
		Unicode2AnsiCat(Data, Name);
		strcat(Data, "=");				/* ANSI */
		Pos->ValLen += strlen(Data);	/* ANSI */
		Data = Pos->ValTbl + Pos->ValLen;
		Pos->ValLen += AnsiCat(Bin, Len, Data) + 1;
	}
	return(SUCCESS);
}


/*-------------------------------------------------------------------------------
Name    :	Unicode2AnsiCat
Desc    :	Convert unicode string to ANSI string and concat string
Param   :	buf		[in/out] ANSI string
			str		[in] unicode string
Return  :	int		length of the ANSI atring
-------------------------------------------------------------------------------*/

static int Unicode2AnsiCat(char *buf, LPTSTR str)
{
	char		*oemStr;	/* not _TCHAR */
	int			length;

	/* convert strings to OEM character set */
	length = WideCharToMultiByte(CP_OEMCP, 0, str, -1, NULL, 0, NULL, NULL);
	oemStr = (char*)Malloc(length+1);
	memset(oemStr, NUL, length+1);
	WideCharToMultiByte(CP_OEMCP, 0, str, -1, oemStr, length, NULL, NULL);
	length = AnsiCat(oemStr, strlen(oemStr), buf);		/* ANSI */
	free(oemStr);
	return(length);
}


/*----- f[^obt@ɒǉ݂ ------------------------------------
*
*	Parameter
*		char *Src : 
*		int len : ̒
*		char *Dst : ݂obt@
*
*	Return Value
*		int ǉoCg
*----------------------------------------------------------------------------*/

static int AnsiCat(char *Src, int Len, char *Dst)		/* ANSI */
{
	int Count;

	Dst += strlen(Dst);			/* ANSI */
	Count = 0;
	for(; Len > 0; Len--)
	{
		if(*Src == '\\')		/* ANSI */
		{
			*Dst++ = '\\';		/* ANSI */
			*Dst++ = '\\';		/* ANSI */
			Count += 2;
		}
		else if((*Src >= 0x20) && (*Src <= 0x7E))
		{
			*Dst++ = *Src;
			Count++;
		}
		else
		{
			sprintf(Dst, "\\%02X", *(unsigned char *)Src);	/* ANSI */
			Dst += 3;
			Count += 3;
		}
		Src++;
	}
	*Dst = NUL;
	return(Count);
}


/*----- obt@ɓǂݍ (ANSI->Unicodeϊ) -------------------------
*
*	Parameter
*		LPTSTR Src : 
*		int Max : obt@̃TCY
*		LPTSTR Dst : ݂obt@
*		multi		}`񂩂ǂ
*
*	Return Value
*		int ǂݍ񂾃oCg
*----------------------------------------------------------------------------*/
static int StrReadIn(char *Src, int Max, LPTSTR Dst, BOOL multi)
{
	int Count;
	int Tmp;
	char *tmpBuf;
	char *pos;
	LPTSTR uniBuf;
	int length;
	int uniLen;

	Max--;
	uniLen = 0;
	*Dst = NUL;
	if(*Src != NUL)
	{
		tmpBuf = Malloc(REG_SECT_MAX);
		pos = tmpBuf;
		Count = 0;
		for(;;)
		{
			if(Count >= REG_SECT_MAX-1)
				break;

			*pos = *Src;
			if(*Src == '\\')
			{
				Src++;
				if(*Src != '\\')
				{
					sscanf(Src, "%02x", &Tmp);
					*pos = Tmp;
					Src++;
				}
			}

			if((*Src == '\0') || (*pos == '\0'))
			{
				length = MultiByteToWideChar(CP_OEMCP, MB_PRECOMPOSED, tmpBuf, -1, NULL, 0);
				uniBuf = (LPTSTR)Malloc((length + 1) * sizeof(_TCHAR));
				memset(uniBuf, 0, (length + 1) * sizeof(_TCHAR));
				MultiByteToWideChar(CP_OEMCP, MB_PRECOMPOSED, tmpBuf, -1, uniBuf, length);
				length = _tcslen(uniBuf);
				if(multi)
				{
					length++;
				}
				if(length > Max)
				{
					break;
				}
				memcpy(Dst + uniLen, uniBuf, length * sizeof(_TCHAR));
				uniLen += length;
				Max -= length;
				*(Dst + uniLen) = NUL;
				free(uniBuf);

				pos = tmpBuf;
				Count = 0;
				if((*Src == '\0') || (Max <= 0) || (*(Src+1) == '\0')) 
				{
					break;
				}
			}
			else
			{
				pos++;
				Count++;
			}
			Src++;
		}
		free(tmpBuf);
	}
	return(uniLen);
}


/*----- obt@ɓǂݍ (ANSÎ܂) --------------------------------
*
*	Parameter
*		LPTSTR Src : 
*		int Max : obt@̃TCY
*		LPTSTR Dst : ݂obt@
*
*	Return Value
*		int ǂݍ񂾃oCg
*----------------------------------------------------------------------------*/
static int StrReadInAnsi(char *Src, int Max, char *Dst)
{
	int Count;
	int Tmp;

	Count = 0;
	while(*Src != NUL)
	{
		if(Count >= Max)
			break;

		if(*Src == '\\')
		{
			Src++;
			if(*Src == '\\')
				*Dst = '\\';
			else
			{
				sscanf(Src, "%02x", &Tmp);
				*Dst = Tmp;
				Src++;
			}
		}
		else
			*Dst = *Src;

		Count++;
		Dst++;
		Src++;
	}
	return(Count);
}


/*----- l ----------------------------------------------------------
*
*	Parameter
*		LPTSTR Handle : nh
*		LPTSTR Name : O
*
*	Return Value
*		LPTSTR lf[^̐擪
*			NULL=w̖O̒lȂ
*----------------------------------------------------------------------------*/
static char *ScanValue(void *Handle, LPTSTR Name)
{
	REGDATATBL *Cur;
	char *Pos;
	char *Ret;
	char		*oemStr;	/* not _TCHAR */
	int			length;

	/* convert strings to OEM character set */
	length = WideCharToMultiByte(CP_OEMCP, 0, Name, -1, NULL, 0, NULL, NULL);
	oemStr = (char*)Malloc(length+1);
	memset(oemStr, NUL, length+1);
	WideCharToMultiByte(CP_OEMCP, 0, Name, -1, oemStr, length, NULL, NULL);

	Ret = NULL;
	Cur = Handle;
	Pos = Cur->ValTbl;
	while(Pos < (Cur->ValTbl + Cur->ValLen))
	{
		if((strncmp(oemStr, Pos, strlen(oemStr)) == 0) &&
		   (*(Pos + strlen(oemStr)) == '='))
		{
			Ret = Pos + strlen(oemStr) + 1;
			break;
		}
		Pos += strlen(Pos) + 1;
	}
	free(oemStr);
	return(Ret);
}


